﻿using System;
using System.Windows.Forms;
using System.Xml.Linq;
using System.IO;

//using System.Xml;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;

namespace Carpeta2Xml
{

    public partial class Form1 : Form
    {

        public Form1()
        {
            InitializeComponent();
        }

        //VARIABLE A NIVEL DE CLASE Para saber si hay alguna ruta seleccionada o no
        bool hayRuta = false;

        // CUADRO DE DIÁLOGO DE SELECCIÓN DE CARPETA
        private void BtExplorar_Click(object sender, EventArgs e)
        {
            try
            {
                // Vaciar el texto de TextBoxXml
                TextBoxXml.Text = "";
                // Instanciar un nuevo FolderBrowserDialog sin botón de crear carpeta nueva
                FolderBrowserDialog ExplorarDlg = new FolderBrowserDialog();
                ExplorarDlg.ShowNewFolderButton = false;
                // Si el usuario selecciona una carpeta, la ruta (SelectedPath) se pasa a TextBoxRuta
                DialogResult result = ExplorarDlg.ShowDialog();
                if (result == DialogResult.OK)
                {
                    TextBoxRuta.Text = ExplorarDlg.SelectedPath;
                    hayRuta = true;
                    // Carpeta raíz del cuadro de diálogo de selección de carpeta: Escritorio arriba del todo
                    ExplorarDlg.RootFolder = Environment.SpecialFolder.Desktop;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error en BtExplorar_Click");
            }
        }

        // CREAR VARIABLES Y LLAMAR A LA FUNCIÓN DIRECTORIO2XML QUE RECORRE CARPETAS, SUBCARPETAS Y ARCHIVOS DEL DIRECTORIO SELECCIONADO
        private void BtXml_Click(object sender, EventArgs e)
        {
            try
            {
                // Salir del procedimiento si el cuadro de texto está vacío y no hay ruta seleccionada
                if (hayRuta == false)
                {
                    return;
                }
                // El nombre de la carpeta inicial se obtiene del cuadro de texto con la ruta, eliminando lo que hay antes de la última aparición del carácter \ para dejar solamente el nombre de la carpeta
                String nombreCarpeta1 = TextBoxRuta.Text.Substring(TextBoxRuta.Text.LastIndexOf("\\") + 1);
                // Texto informativo en la ventana del formulario
                this.Text = "Generando XML para el directorio " + nombreCarpeta1;
                //
                // Clase DirectoryInfo para obtener información del directorio inicial (raíz)
                DirectoryInfo infoCarpeta1 = new DirectoryInfo(TextBoxRuta.Text);
                // XDocument representa un documento XML que se va a obtener desde la función Directorio2Xml a la que se le pasa como parámetro el directorio inicial (raíz)
                XDocument XDocumento = new XDocument(Directorio2Xml(infoCarpeta1));
                // Cadena para contener el texto XML devuelto al terminar la función Directorio2Xml
                String TextoXml;
                TextoXml = XDocumento.ToString();
                //
                // Cadenas para colocar al principio del texto la definición del documento XML
                String nodoCabecera = "<xml version=\"1.0\" encoding=\"UTF-8\">";
                // Escribir al principio del cuadro de texto la línea con nodoCabecera y después el texto XML completo terminado con </xml> para finalizar la etiqueta
                TextBoxXml.Text = nodoCabecera + "\r\n" + TextoXml + "\r\n" + "</xml>";
                // Variable hayRuta a false para que no actúe el botón XML hasta seleccionar carpeta de nuevo con el diálogo
                hayRuta = false;
                // Cadena para contener todo el contenido actual del cuadro de texto
                String XmlCompleto = TextBoxXml.Text;
                // Ruta a la carpeta Esritorio del usuario
                String archivoDesktop = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
                // El nombre del archivo es el del directorio raíz con extensión .xml guardado en el Escritorio
                archivoDesktop += "\\" + nombreCarpeta1 + ".xml";
                // File.WriteAllText crea un archivo nuevo, escribe el contenido en el archivo y lo cierra. Si el archivo de destino ya existe, se sobrescribe
                File.WriteAllText(archivoDesktop, TextBoxXml.Text);
                // Texto informativo en la ventana del formulario
                this.Text = "XML guardado en " + archivoDesktop;
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message, "Error en BtXml_Click");

                if (ex is UnauthorizedAccessException || ex is InvalidOperationException)
                {
                    MessageBox.Show("Hay un error de acceso no autorizado \"UnauthorizedAccessException\" o de operación no permitida \"InvalidOperationException\".\n\nElige un directorio sin archivos de sistema para evitar este problema.", "Error en BtXml_Click");
                }
                else
                {
                    MessageBox.Show(ex.Message, "Error en BtXml_Click");
                }



            }
        }

        // FUNCIÓN QUE RECORRE CARPETA, SUBCARPETAS Y ARCHIVOS AÑADIENDO AL TEXTO XML
        // - ELEMENTOS XML (xelement) CARPETA / ARCHIVO
        // - ATRIBUTO (xattribute) EL NOMBRE DE LA CARPETA O EL ARCHIVO
        // Se pueden añadir más atributos: tamaño (length), etc.
        public static XElement Directorio2Xml(DirectoryInfo directorio)
        {
            try
            {
                // Los elementos XML son carpeta y archivo y el atributo de ambos es nombre
                XElement elementoXml = new XElement("carpeta", new XAttribute("nombre", directorio.Name));
                // recorrer archivos del directorio
                foreach (var archivo in directorio.GetFiles())
                    elementoXml.Add(new XElement("archivo", new XAttribute("nombre", archivo.Name)));
                // recorrer subdirectorios del directorio
                foreach (var subCarpeta in directorio.GetDirectories())
                    elementoXml.Add(Directorio2Xml(subCarpeta));
                // Devuelve la información recopilada en formato XML
                return elementoXml;
            }
            catch (Exception)
            {
                // La compilación (cuando Release está activa) se interrumpe en la función Directorio2Xml con el mensaje: "Error CS0161 'Form1.Directorio2Xml(DirectoryInfo)': no todas las rutas de acceso de código devuelven un valor". Se puede evitar lanzando con throw la excepción InvalidOperationException en lugar de capturarla con catch
                throw new InvalidOperationException();
            }
        }

        // CERRAR EL FORMULARIO
        private void BtCerrar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

    }
}
